@@ -7,8 +7,8 @@ catch_entries_commun = function(K_stages, N_subsets, f, ratio_Delta_star_d1, ord
     K_stages = as.integer(K_stages)
     print(paste("The number of stages was transformed into integer: K_stages=", K_stages, sep=""))
   }
-  if(K_stages<2){
-    stop("The number of stages must be > 1")
+  if(K_stages<1){
+    stop("The number of stages must be > 0")
   }
   if(as.integer(N_subsets) != N_subsets){
     N_subsets = as.integer(N_subsets)
@@ -34,7 +34,7 @@ catch_entries_commun = function(K_stages, N_subsets, f, ratio_Delta_star_d1, ord
   } 
   if(as.integer(seed) != seed){
     seed = as.integer(seed)
-    print(paste("The mseed was transformed into integer: seed=", seed, sep=""))
+    print(paste("The seed was transformed into integer: seed=", seed, sep=""))
   }
   return(list(K_stages, N_subsets, f, ratio_Delta_star_d1, ordering, increasing_theta, seed))
 }
@@ -99,7 +99,7 @@ catch_entries_FI = function(K_stages, N_subsets, f, ratio_Delta_star_d1, l, u, t
 
 catch_entries_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, type_outcome, param_outcome=NA,
                          n_max=NA, incl_rate=NA, cens_rate=NA, param_cens=NA, med_cur_c=NA, HR=NA, 
-                         nb_required=NA, ordering, increasing_theta=FALSE, nsim=1000, seed=42){
+                         nb_required=NA, nmax_wait=+Inf, ordering, increasing_theta=FALSE, nsim=1000, seed=42){
   catch = catch_entries_commun(K_stages, N_subsets, f, ratio_Delta_star_d1, ordering, increasing_theta, seed)
   K_stages = catch[[1]]
   N_subsets = catch[[2]]
@@ -195,6 +195,9 @@ catch_entries_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, t
   if(type_outcome=="survival" && is.na(nb_required)){
     stop("The maximum number of events required, nb_required, must be provided")
   }
+  if(type_outcome=="survival" && is.na(nmax_wait)== FALSE && (nmax_wait < nb_required)){
+    stop("The maximum number of patients to include in the trial must be superior or equal to the number of events required")
+  }
   if(type_outcome=="survival" && (cens_rate < 0 || cens_rate > 1)){
     stop("The drop-out rate, cens_rate, must be between 0 and 1")
   }
@@ -215,7 +218,7 @@ catch_entries_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, t
     print(paste("The number of simulations to perform was transformed into integer: nsim=", nsim, sep=""))
   }
   return(list(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, type_outcome, param_outcome, n_max, incl_rate, 
-              cens_rate, param_cens, med_cur_c, HR, nb_required, ordering, increasing_theta, nsim, seed))
+              cens_rate, param_cens, med_cur_c, HR, nb_required, ordering, increasing_theta, nsim, seed, nmax_wait))
 }
 
 
@@ -271,7 +274,7 @@ stage_1_evaluation = function(keep, Z_1j, f, u){
     f_keep = f[keep] 
     f_S = sum(f_keep)
     Z_1S = 1/sqrt(f_S) * sum( Z_1j[keep] * sqrt(f_keep) )
-    if(Z_1S >= u[1]){
+    if(Z_1S > u[1]){
       return(list("stage"=1,"S"=keep))
     }
     else{
@@ -541,7 +544,7 @@ max_FI = function(K_stages, N_subsets, f, ratio_Delta_star_d1, l, u, type_outcom
 
 #########
 ### Application of Magnusson and Turnbull with data
-magnusson_turnbull = function(stage_cur, keep=NA, N_subsets, f, Y, I, l, u, ratio_Delta_star_d1, ordering, increasing_theta=FALSE){
+magnusson_turnbull = function(stage_cur, keep=NA, N_subsets, Y, I, l, u, ordering, increasing_theta=FALSE){
   Z = Y/sqrt(I)
   if(stage_cur==0){
     keep = stage_1_selection(N_subsets, Z, l, ordering, increasing_theta)
@@ -595,53 +598,101 @@ in_list = function(vec,liste){
 
 #########
 ### Simulation of patients' inclusions for survival data
-incl_and_update_patients = function(incl_rate, duration, follow, time_event, trt, biom_group, lost_censor, time_cens, keep, f, cens_rate, param_cens, med_cur_c, HR){
+incl_and_update_patients = function(incl_rate, duration, follow, time_event, trt, biom_group, lost_censor, time_cens, keep, f, 
+cens_rate, param_cens, med_cur_c, HR, time_event_study, nb_event_1ns, nb_required, nmax_wait, dur_incl){
+  # Time until new inclusion
   if(length(incl_rate) == 1){
     new_incl = rexp(1,incl_rate)
   }
   else if(length(incl_rate) > 1){
     new_incl = rexp(1,sum(incl_rate))
   }
-  bg = sample(1:length(f), 1, prob=f)
-  if(is.element(bg,keep)){
-    biom_group = c(biom_group, bg)
-    tc = rbinom(1,1,0.5) 
-    trt = c(trt, tc)
-    if(tc == 0){
-      te = rexp( 1, 1/med_cur_c )
-    }
-    else{
-      te = rexp( 1, 1/(med_cur_c/HR[bg]) ) 
-    }
-    censor = rbinom(1,1,cens_rate)
-    lost_censor = c(lost_censor, censor)
-    if(censor==1){
-      te = +Inf
-      tcens = rexp(1,1/param_cens)
-      time_cens = c(time_cens, tcens)
-    }
-    else{
-      time_cens = c(time_cens, +Inf)
-    }
-    time_event = c(time_event, te)
-    follow = c(follow,0) 
+  
+  follow_temp = follow+new_incl
+  follow_temp = pmin(follow_temp, time_cens)
+  ind_event_temp = (time_event <= follow_temp)
+  ind_group_S =c()
+  for(j in keep){
+    ind_group_S = c(ind_group_S, which(biom_group==j))
   }
-  follow = pmin(follow+new_incl, time_cens)
-  duration = duration + new_incl
-  ind_event = (time_event <= follow)
-  time_min = pmin(time_event, follow, na.rm=TRUE)
-  nb_events = sum(ind_event)
-  return(list(follow,duration,trt,biom_group,time_event,ind_event,time_min,nb_events,lost_censor,time_cens))
+  nb_events_S_temp = sum(ind_event_temp[ind_group_S])
+  # If number of events and number of patients not achieved OR
+  # if we achieved the maximum number of patients and not enough patients for events
+  # then we include a new patient
+  if( nb_event_1ns+nb_events_S_temp < nb_required && 
+     (length(trt) < nmax_wait ||
+      (length(trt) == nmax_wait && nb_event_1ns+length(trt[ind_group_S]) < nb_required))
+     ){
+    duration = duration + new_incl
+    follow = follow+new_incl
+    
+    bg = sample(1:length(f), 1, prob=f)
+    # we include the patient only if he is in the subgroup retained
+    if(is.element(bg,keep)){
+      biom_group = c(biom_group, bg)
+      tc = rbinom(1,1,0.5) 
+      trt = c(trt, tc)
+      if(tc == 0){
+        te = rexp( 1, 1/med_cur_c )
+      }
+      else{
+        te = rexp( 1, 1/(med_cur_c/HR[bg]) ) 
+      }
+      censor = rbinom(1,1,cens_rate)
+      lost_censor = c(lost_censor, censor)
+      if(censor==1){
+        te = +Inf
+        tcens = rexp(1,1/param_cens)
+        time_cens = c(time_cens, tcens)
+      }
+      else{
+        time_cens = c(time_cens, +Inf)
+      }
+      time_event = c(time_event, te)
+      follow = c(follow,0)
+      time_event_study = c(time_event_study, duration+te)
+    }
+    
+    follow = pmin(follow, time_cens)
+    ind_event = (time_event <= follow)
+    time_min = pmin(time_event, follow, na.rm=TRUE)
+    nb_events = sum(ind_event)
+  }
+  else{
+    dur_incl = duration
+    # Otherwise, if we achieved (or more) the number of events required OR
+    # if the maximum number of patients is achieved and must wait (and enough patients for events)
+    if( (nb_event_1ns+nb_events_S_temp >= nb_required) ||
+        (nb_event_1ns+nb_events_S_temp < nb_required && length(trt) == nmax_wait && 
+         nb_event_1ns+length(trt[ind_group_S]) >= nb_required)
+      ){
+      time_event_study_S = time_event_study[ind_group_S]
+      ord_ev = order(time_event_study_S)
+      ordered_time_event_study = time_event_study_S[ord_ev]
+      time_ev_req = ordered_time_event_study[nb_required-nb_event_1ns]
+      diff_time = time_ev_req - duration+0.00001
+      duration = duration + diff_time
+      follow = follow + diff_time
+      follow = pmin(follow, time_cens)
+      ind_event = (time_event <= follow)
+      time_min = pmin(time_event, follow, na.rm=TRUE)
+      nb_events = sum(ind_event)
+    }
+  }
+  
+  return(list(follow,duration,trt,biom_group,time_event,ind_event,time_min,nb_events,lost_censor,
+              time_cens,time_event_study,dur_incl))
 }
 
 
+
 #########
 ### Simulation of one design with Magnusson and Turnbull for survival data 
 sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl_rate, cens_rate, param_cens, 
-                         med_cur_c, HR, nb_required, ordering, increasing_theta=FALSE){
+                         med_cur_c, HR, nb_required, nmax_wait=+Inf, ordering, increasing_theta=FALSE){
   reject = NA
   nb_events = 0
-  duration = rexp(1,incl_rate)
+  duration = 0
   follow = c()
   time_event = c()
   ind_event = c()
@@ -650,14 +701,19 @@ sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl
   time_min = c()
   lost_censor = c()
   time_cens = c()
+  time_event_study = c()
+  dur_incl = NA
   
-  n_1 = round(nb_required / (1+sum(ratio_Delta_star_d1)))
-  #n_req_step = c(n_1, n_1*ratio_Delta_star_d1)
+  n_1 = ceiling(nb_required / (1+sum(ratio_Delta_star_d1)))
   n_req_step = c(n_1, nb_required-n_1)
-  
+
   #Step 1
   while(nb_events < n_req_step[1]){
-    incl_up = incl_and_update_patients(incl_rate, duration, follow, time_event, trt, biom_group, lost_censor, time_cens, 1:N_subsets, f, cens_rate, param_cens, med_cur_c, HR)  
+    incl_up = incl_and_update_patients(incl_rate=incl_rate, duration=duration, follow=follow, time_event=time_event, 
+                                       trt=trt, biom_group=biom_group, lost_censor=lost_censor, time_cens=time_cens, 
+                                       keep=1:N_subsets, f=f, cens_rate=cens_rate, param_cens=param_cens, 
+                                       med_cur_c=med_cur_c, HR=HR, time_event_study=time_event_study, nb_event_1ns=0, 
+                                       nb_required=n_req_step[1], nmax_wait=nmax_wait, dur_incl=dur_incl)
     follow = incl_up[[1]]
     duration = incl_up[[2]]
     trt = incl_up[[3]]
@@ -668,6 +724,8 @@ sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl
     nb_events = incl_up[[8]]
     lost_censor = incl_up[[9]]
     time_cens = incl_up[[10]]
+    time_event_study = incl_up[[11]]
+    dur_incl = incl_up[[12]]
   }
   nb_events_stage1 = nb_events
   Y_1j = numeric(N_subsets)
@@ -679,18 +737,19 @@ sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl
       temp = survdiff(Surv(time_min[ind_group_j], ind_event[ind_group_j]) ~ trt[ind_group_j])
       Z_1j = (temp$obs[1]-temp$exp[1])/sqrt(temp$var[1,1])
       I_1j[j] = nb_event_j/4
-      Y_1j[j] = Z_1j * sqrt(I_1j[j])
+      Y_1j[j] = Z_1j*sqrt(I_1j[j])
     }
     else{
-      Y_1j[j] = +Inf
+      Y_1j[j] = NA
       I_1j[j] = 1
     }
   }
   k=1
-  selection = magnusson_turnbull(0, NA, N_subsets, f, Y_1j, I_1j, l, u, ratio_Delta_star_d1, ordering, increasing_theta)
+  selection = magnusson_turnbull(0, NA, N_subsets, Y_1j, I_1j, l, u, ordering, increasing_theta)
   if(selection$Acceptation == 1){
     reject = 0
     keep = c()
+    dur_incl = duration
   }
   else{
     keep = selection$Keep
@@ -699,29 +758,34 @@ sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl
       ind_group_1S = c(ind_group_1S, which(biom_group==j))
     }
     nb_event_1S = sum(ind_event[ind_group_1S])
-    nb_events_stage1S = nb_event_1S
-    if(length(which(trt[ind_group_1S]==0))>0 && length(which(trt[ind_group_1S]==1))>0 && nb_event_1S>0){
+    nb_events_1nS = nb_events_stage1 - nb_event_1S
+    if(length(which(trt[ind_group_1S]==0))>1 && length(which(trt[ind_group_1S]==1))>1 && nb_event_1S>0){
       temp = survdiff(Surv(time_min[ind_group_1S], ind_event[ind_group_1S]) ~ trt[ind_group_1S])
       Zp_1S = (temp$obs[1]-temp$exp[1])/sqrt(temp$var[1,1])
       I_1S = nb_event_1S/4
-      Y_1S = Zp_1S * sqrt(I_1S)
+      Y_1S = Zp_1S*sqrt(I_1S)
     }
     else{
-      Y_1S = -Inf
+      Y_1S = NA
       I_1S = 1
     }
-    step1 = magnusson_turnbull(1, keep, N_subsets, f, Y_1S, I_1S, l, u, ratio_Delta_star_d1, ordering, increasing_theta) 
+    step1 = magnusson_turnbull(1, keep, N_subsets, Y_1S, I_1S, l, u, ordering, increasing_theta) 
     if(step1$Rejection == 1){
       reject = 1
+      dur_incl = duration
     }  
     else{  
       k = 2
-      f_keep = f[keep]
-      f_S = sum(f_keep)
-      nb_event_kS = nb_events_stage1S
-      while(k <= K_stages && is.na(reject)){ 
-        while((nb_events_stage1-nb_events_stage1S)+nb_event_kS < cumsum(n_req_step)[k]){
-          incl_up = incl_and_update_patients(incl_rate, duration, follow, time_event, trt, biom_group, lost_censor, time_cens, keep, f, cens_rate, param_cens, med_cur_c, HR)  
+      nb_event_kS = nb_event_1S
+      while(k <= K_stages && is.na(reject)){
+        while(nb_events_1nS+nb_event_kS < cumsum(n_req_step)[k]){
+          incl_up = incl_and_update_patients(incl_rate=incl_rate, duration=duration, follow=follow, 
+                                             time_event=time_event, trt=trt, biom_group=biom_group, 
+                                             lost_censor=lost_censor, time_cens=time_cens, keep=keep, f=f, 
+                                             cens_rate=cens_rate, param_cens=param_cens, med_cur_c=med_cur_c, 
+                                             HR=HR, time_event_study=time_event_study, nb_event_1ns=nb_events_1nS, 
+                                             nb_required=cumsum(n_req_step)[k], nmax_wait=nmax_wait, 
+                                             dur_incl=dur_incl)
           follow = incl_up[[1]]
           duration = incl_up[[2]]
           trt = incl_up[[3]]
@@ -732,17 +796,30 @@ sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl
           nb_events = incl_up[[8]]
           lost_censor = incl_up[[9]]
           time_cens = incl_up[[10]]
+	        time_event_study = incl_up[[11]]
+          dur_incl = incl_up[[12]]
           ind_group_S =c()
           for(j in keep){
             ind_group_S = c(ind_group_S, which(biom_group==j))
           }
           nb_event_kS = sum(ind_event[ind_group_S])
         }
-        temp = survdiff(Surv(time_min[ind_group_S], ind_event[ind_group_S]) ~ trt[ind_group_S])
-        Zp_kS = (temp$obs[1]-temp$exp[1])/sqrt(temp$var[1,1])
-        I_kS = nb_event_kS/4
-        Y_kS = Zp_kS * sqrt(I_kS)
-        stepk = magnusson_turnbull(k, keep, N_subsets, f, Y_kS, I_kS, l, u, ratio_Delta_star_d1, ordering, increasing_theta)
+        ind_group_S =c()
+        for(j in keep){
+          ind_group_S = c(ind_group_S, which(biom_group==j))
+        }
+        nb_event_kS = sum(ind_event[ind_group_S])
+        if(length(which(trt[ind_group_S]==0))>1 && length(which(trt[ind_group_S]==1))>1 && nb_event_kS>0){
+          temp = survdiff(Surv(time_min[ind_group_S], ind_event[ind_group_S]) ~ trt[ind_group_S])
+          Zp_kS = (temp$obs[1]-temp$exp[1])/sqrt(temp$var[1,1])
+          I_kS = nb_event_kS/4
+          Y_kS = Zp_kS*sqrt(I_kS)
+        }
+        else{
+          Y_kS = NA
+          I_kS = 1
+        }
+        stepk = magnusson_turnbull(k, keep, N_subsets, Y_kS, I_kS, l, u, ordering, increasing_theta)
         if(stepk$Rejection == 1){
           reject = 1  
         }
@@ -755,14 +832,14 @@ sim_one_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl
       }
     }
   }
-  return(list("reject"=reject, "keep"=keep, "stage"=k, "duration"=duration, "nb_patients"=length(trt)))
+  return(list("reject"=reject, "keep"=keep, "stage"=k, "duration"=duration, "nb_patients"=length(trt),"dur_incl"=dur_incl))
 }
 
 
 #########
 ### Simulation of n design with Magnusson and Turnbull for survival data
-sim_trials_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl_rate, cens_rate, param_cens, 
-                            med_cur_c, HR, nb_required, ordering, increasing_theta=FALSE, nsim=1000, seed=42){
+sim_trials_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl_rate, cens_rate, param_cens, med_cur_c, 
+                            HR, nb_required, nmax_wait=+Inf, ordering, increasing_theta=FALSE, nsim=1000, seed=42){
   set.seed(seed) 
   prob_rejec = 0
   prob_accep = 0
@@ -775,12 +852,20 @@ sim_trials_OS_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, i
   accep_stage = numeric(K_stages)
   mean_duration = 0
   mean_pat = 0
+  mean_dur_incl = 0
+  dist_pat = c()
+  dist_duration = c()
+  dist_dur_incl = c()
+  quant_pat = c()
+  quant_duration = c()
+  quant_dur_incl = c()
+  
   for(isim in 1:nsim){
 if(isim %% 1000 == 0){
-    print(isim)
+print(isim)
 }
     sMT = sim_one_OS_MT(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl_rate, cens_rate, param_cens, 
-                        med_cur_c, HR, nb_required, ordering, increasing_theta) 
+                        med_cur_c, HR, nb_required, nmax_wait, ordering, increasing_theta) 
     reject = sMT$reject
     st = sMT$stage
     if(reject == 1){
@@ -833,6 +918,12 @@ if(isim %% 1000 == 0){
     }
     mean_duration = mean_duration + sMT$duration
     mean_pat = mean_pat + sMT$nb_patients
+    if(is.infinite(nmax_wait)==FALSE){
+      mean_dur_incl = mean_dur_incl + sMT$dur_incl
+      dist_pat = c(dist_pat, sMT$nb_patients)
+      dist_duration = c(dist_duration, sMT$duration)
+      dist_dur_incl = c(dist_dur_incl, sMT$dur_incl)
+    }
   }
   prob_rejec = prob_rejec/nsim
   prob_accep = prob_accep/nsim
@@ -844,10 +935,16 @@ if(isim %% 1000 == 0){
   accep_stage = accep_stage/nsim*100
   mean_duration = mean_duration/nsim
   mean_pat = mean_pat/nsim
+  mean_dur_incl = mean_dur_incl/nsim
+  quant_pat = quantile(dist_pat, probs =c(0.5,0.75,1))
+  quant_duration = quantile(dist_duration, probs =c(0.5,0.75,1))
+  quant_dur_incl = quantile(dist_dur_incl, probs =c(0.5,0.75,1))
   
   return(list("prob_rejec"=prob_rejec, "prob_accep"=prob_accep, "list_keep"=list_keep, "pct_keep"=pct_keep,
               "pct_rejec_keep"=pct_rejec_keep, "pct_rejec_keep1"=pct_rejec_keep1, "pct_rejec_keep2"=pct_rejec_keep2,
-              "rejec_stage"=rejec_stage, "accep_stage"=accep_stage, "mean_duration"=mean_duration, "mean_pat"=mean_pat))
+              "rejec_stage"=rejec_stage, "accep_stage"=accep_stage, "mean_duration"=mean_duration, "mean_pat"=mean_pat,
+              "mean_dur_incl"=mean_dur_incl, "dist_pat"=dist_pat, "dist_duration"=dist_duration, "dist_dur_incl"=dist_dur_incl,
+              "quant_pat"=quant_pat, "quant_duration"=quant_duration, "quant_dur_incl"=quant_dur_incl))
 }
 
 
@@ -862,10 +959,16 @@ test_BC = function(ind_trt_group_j, ind_con_group_j, outcome, type_outcome){
     mean2 = mean(outcome_trt2)
     if(n1 > 1 && n2 > 1){
       if(type_outcome=="binary"){
-        prop_commune = (mean1*n1+mean2*n2)/(n1+n2)
-        var_pool = prop_commune*(1-prop_commune)
-        Z_1j = (mean1-mean2) / sqrt( var_pool * (1/n1 + 1/n2) )
-        I_1j = (n1+n2) / (4*mean(c(outcome_trt1,outcome_trt2))*(1-mean(c(outcome_trt1,outcome_trt2))))
+        if( mean1 == mean2 && (mean1 == 0 || mean1 == 1) ){
+          Z_1j = NA
+          I_1j = 1
+        }
+        else{
+          prop_commune = (mean1*n1+mean2*n2)/(n1+n2)
+          var_pool = prop_commune*(1-prop_commune)
+          Z_1j = (mean1-mean2) / sqrt( var_pool * (1/n1 + 1/n2) )
+          I_1j = (n1+n2) / (4*mean(c(outcome_trt1,outcome_trt2))*(1-mean(c(outcome_trt1,outcome_trt2))))
+        }
       }
       else if(type_outcome=="continuous"){ 
         var1 = var(outcome_trt1)
@@ -884,7 +987,7 @@ test_BC = function(ind_trt_group_j, ind_con_group_j, outcome, type_outcome){
 
 
 #########
-### Simulation of one design with Magnusson and Turnbull for survival data 
+### Simulation of one design with Magnusson and Turnbull for binary or continuous data
 sim_one_BC_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_max, type_outcome, param_outcome, ordering, increasing_theta=FALSE){
   reject = NA
   outcome = c()
@@ -892,7 +995,7 @@ sim_one_BC_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_ma
   biom_group = c()
   n_pat = 0
   
-  n_1 = round(n_max / (1+sum(ratio_Delta_star_d1)))
+  n_1 = ceiling(n_max / (1+sum(ratio_Delta_star_d1)))
   #n_step = c(n_1, n_1*ratio_Delta_star_d1)
   n_step = c(n_1, n_max-n_1)
   
@@ -918,13 +1021,13 @@ sim_one_BC_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_ma
     t_stat = test_BC(ind_trt_group_j, ind_con_group_j, outcome, type_outcome)
     I_1j[j] = t_stat[2]
     Y_1j[j] = t_stat[1] * sqrt(I_1j[j])
-    if(is.na(Y_1j[j]) || Y_1j[j]==+Inf){
-      Y_1j[j] = +Inf
+    if(is.na(Y_1j[j])){
+      Y_1j[j] = -Inf
       I_1j[j] = 1
     }
   }
   k=1
-  selection = magnusson_turnbull(0, NA, N_subsets, f, Y_1j, I_1j, l, u, ratio_Delta_star_d1, ordering, increasing_theta)
+  selection = magnusson_turnbull(0, NA, N_subsets, Y_1j, I_1j, l, u, ordering, increasing_theta)
   if(selection$Acceptation == 1){
     reject = 0
     keep = c()
@@ -938,11 +1041,11 @@ sim_one_BC_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_ma
     t_statS = test_BC(intersect(ind_group_1S,which(trt==1)), intersect(ind_group_1S,which(trt==0)), outcome, type_outcome)
     I_1S = t_statS[2]
     Y_1S = t_statS[1] * sqrt(I_1S)
-    if(is.na(Y_1S) || Y_1S==+Inf){
+    if(is.na(Y_1S)){
       Y_1S = -Inf
       I_1S = 1
     }
-    step1 = magnusson_turnbull(1, keep, N_subsets, f, Y_1S, I_1S, l, u, ratio_Delta_star_d1, ordering, increasing_theta) 
+    step1 = magnusson_turnbull(1, keep, N_subsets, Y_1S, I_1S, l, u, ordering, increasing_theta) 
     if(step1$Rejection == 1){
       reject = 1
     }  
@@ -978,7 +1081,7 @@ sim_one_BC_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_ma
         t_statkS = test_BC(intersect(ind_group_S,which(trt==1)), intersect(ind_group_S,which(trt==0)), outcome, type_outcome)
         I_kS = t_statkS[2]
         Y_kS = t_statkS[1] * sqrt(I_kS)
-        stepk = magnusson_turnbull(k, keep, N_subsets, f, Y_kS, I_kS, l, u, ratio_Delta_star_d1, ordering, increasing_theta)
+        stepk = magnusson_turnbull(k, keep, N_subsets, Y_kS, I_kS, l, u, ordering, increasing_theta)
         if(stepk$Rejection == 1){
           reject = 1  
         }
@@ -1010,10 +1113,11 @@ sim_trials_BC_MT = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n
   rejec_stage = numeric(K_stages)
   accep_stage = numeric(K_stages)
   mean_pat = 0
+  dist_pat = c()
 
   for(isim in 1:nsim){
 if(isim %% 1000 == 0){
-    print(isim)
+print(isim)
 }
     sMT = sim_one_BC_MT(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_max, type_outcome, param_outcome, ordering, increasing_theta)
     reject = sMT$reject
@@ -1067,6 +1171,7 @@ if(isim %% 1000 == 0){
       }
     }
     mean_pat = mean_pat + sMT$nb_patients
+    dist_pat = c(dist_pat, sMT$nb_patients)
   }
   prob_rejec = prob_rejec/nsim
   prob_accep = prob_accep/nsim
@@ -1080,7 +1185,7 @@ if(isim %% 1000 == 0){
   
   return(list("prob_rejec"=prob_rejec, "prob_accep"=prob_accep, "list_keep"=list_keep, "pct_keep"=pct_keep,
               "pct_rejec_keep"=pct_rejec_keep, "pct_rejec_keep1"=pct_rejec_keep1, "pct_rejec_keep2"=pct_rejec_keep2,
-              "rejec_stage"=rejec_stage, "accep_stage"=accep_stage, "mean_pat"=mean_pat))
+              "rejec_stage"=rejec_stage, "accep_stage"=accep_stage, "mean_pat"=mean_pat, "dist_pat"=dist_pat))
 }
 
 
@@ -1091,9 +1196,9 @@ if(isim %% 1000 == 0){
 ### General function to simulate n design with Magnusson and Turnbull for a given type of outcome
 sim_magnusson_turnbull = function(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, type_outcome, param_outcome=NA,
                                   n_max=NA, incl_rate=NA, cens_rate=NA, param_cens=NA, med_cur_c=NA, HR=NA, 
-                                  nb_required=NA, ordering, increasing_theta=FALSE, nsim=1000, seed=42){
+                                  nb_required=NA, nmax_wait=+Inf, ordering, increasing_theta=FALSE, nsim=1000, seed=42){
   catch = catch_entries_MT(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, type_outcome, param_outcome, n_max, incl_rate, 
-                        cens_rate, param_cens, med_cur_c, HR, nb_required, ordering, increasing_theta, nsim, seed)
+                        cens_rate, param_cens, med_cur_c, HR, nb_required, nmax_wait, ordering, increasing_theta, nsim, seed)
   K_stages = catch[[1]]
   N_subsets = catch[[2]]
   f = catch[[3]]
@@ -1113,10 +1218,11 @@ sim_magnusson_turnbull = function(K_stages, N_subsets, f, l, u, ratio_Delta_star
   increasing_theta = catch[[17]]
   nsim = catch[[18]]
   seed = catch[[19]]
+  nmax_wait = catch[[20]]
     
   if(type_outcome=="survival"){
     return(sim_trials_OS_MT(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, incl_rate, cens_rate, param_cens, 
-                                med_cur_c, HR, nb_required, ordering, increasing_theta, nsim, seed))  
+                                med_cur_c, HR, nb_required, nmax_wait, ordering, increasing_theta, nsim, seed))  
   }
   else if(type_outcome=="binary" || type_outcome=="continuous"){
     return(sim_trials_BC_MT(K_stages, N_subsets, f, l, u, ratio_Delta_star_d1, n_max, type_outcome, param_outcome, 
